@@ -3,10 +3,14 @@
 import json
 import os
 from typing import Callable, Dict
+from warnings import warn
+
+import numpy as np
 
 from zod.eval.detection._nuscenes_eval.common.data_classes import EvalBoxes
 from zod.eval.detection._nuscenes_eval.common.utils import center_distance
 from zod.eval.detection._nuscenes_eval.detection.algo import accumulate, calc_ap, calc_tp
+from zod.eval.detection._nuscenes_eval.detection.constants import DETECTION_NAMES
 from zod.eval.detection._nuscenes_eval.detection.data_classes import (
     DetectionBox,
     DetectionConfig,
@@ -58,8 +62,12 @@ def evaluate_nuscenes_style(
     det_boxes: EvalBoxes,
     verbose: bool = False,
     output_path: str = None,
+    verify_coordinate_system: bool = True,
 ) -> Dict[str, float]:
     """Perform nuscenes evaluation based on a number of groundtruths and detections."""
+    if verify_coordinate_system:
+        _check_coordinate_system(gt_boxes)
+
     detection_cfg = DetectionConfig(**NUSCENES_DEFAULT_SETTINGS)
     detection_metrics = DetectionMetrics(detection_cfg)
 
@@ -102,6 +110,24 @@ def evaluate_nuscenes_style(
     return serialized
 
 
+def _check_coordinate_system(gt_boxes: EvalBoxes):
+    """Use heuristics to check if the boxes were provided in the ego coordinate system."""
+    # The key assumption is x axis has the longest range in the ego coordinate system
+    # whereas for camera and lidar the z and y axis are the longest range respectively
+    centers = np.array([box.translation for box in gt_boxes.all])
+    center_means = np.mean(centers, axis=0)
+    max_avgdist_axis = np.argmax(center_means)
+    error_msg = (
+        "Looks like the boxes were provided in the {sensor} coordinate system. "
+        "Please convert them to the ego coordinate system. If you know what you are doing, "
+        " you can disable this check by setting `verify_coordinate_system=False`."
+    )
+    if max_avgdist_axis == 1:
+        raise ValueError(error_msg.format(sensor="lidar"))
+    elif max_avgdist_axis == 2:
+        raise ValueError(error_msg.format(sensor="camera"))
+
+
 def _nuscenes_evaluate(
     gt_boxes: EvalBoxes,
     det_boxes: EvalBoxes,
@@ -204,3 +230,32 @@ def _serialize(detection_metrics: DetectionMetrics) -> Dict[str, float]:
         **class_aps,
         **class_tps,
     }
+
+
+if __name__ == "__main__":
+    # instantiate ZOD
+    from zod import AnnotationProject, ZodFrames
+
+    zod = ZodFrames("/Users/s0000960/data/zod", "mini")
+    gt_boxes = EvalBoxes()
+    for sample in zod:
+        od_annos = sample.get_annotation(AnnotationProject.OBJECT_DETECTION)
+        boxes = []
+        for obj in od_annos:
+            if obj.name not in DETECTION_NAMES:
+                continue
+            if obj.box3d is None:
+                continue
+            boxes.append(
+                DetectionBox(
+                    sample_token=sample.info.id,
+                    translation=tuple(obj.box3d.center),
+                    size=tuple(obj.box3d.size),
+                    rotation=tuple(obj.box3d.orientation.elements),
+                    num_pts=10,
+                    detection_name=obj.name,
+                    detection_score=-1.0,  # ground truth boxes have no score
+                )
+            )
+        gt_boxes.add_boxes(sample.info.id, boxes)
+    evaluate_nuscenes_style(gt_boxes, gt_boxes, verbose=True, verify_coordinate_system=False)
